# -------------------------------------------------------------------------------------
# flo::editor::Headlines
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2002
#
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden
#
#    Description:
#
#      Headlines content component. Should let the user add hyperlinks to pages
#
# -------------------------------------------------------------------------------------
package flo::editor::Headlines;
use flo::Record::Editor;
use Text::Unidecode;
use MKDoc::Config;
use flo::Editor;
use flo::Standard qw /raw_path_info session table template current_document/;
use MKDoc::CGI;
use MKDoc::Util::Text2HTML;
use MKDoc::XML::Dumper;
use Unicode::Normalize;
use strict;
use utf8;

use constant DEFAULT_BASE_PATH     => '/';
use constant DEFAULT_MAX_HEADLINES => 7;


use base qw /flo::Component
	     flo::editor::Mixin::compute_name
	     flo::editor::Mixin::normalize_name/;


sub preferred_extension { 'html' };


sub base_document
{
    my $self = shift;
    my $document_t = flo::Standard::table ('Document');
    return $document_t->get ( Full_Path => $self->from_path() );
}


##
# $class->new;
# ------------
#   Instanciates a new flo::Editor::Headlines component object
##
sub new
{
    my $class = shift;
    $class    = ref $class || $class;
    my $self  = bless { @_ }, $class;
    
    my $cgi = $self->{cgi};
    my $param_name = $self->{param_name};
    
    $self->{title}         = '';
    $self->{from_path}     = flo::Standard::current_document()->path();
    $self->{max_headlines} = DEFAULT_MAX_HEADLINES;
    if (defined $cgi and not defined $cgi->param ($param_name . "_add"))
    {
	my $title = $cgi->param          ($param_name . "_title")     || '';
	my $path  = $cgi->param          ($param_name . "_from_path") || flo::Standard::current_document()->path();
	my $default_number = $cgi->param ($param_name . "_number")    || DEFAULT_MAX_HEADLINES;
	$self->set_title ($title);
	$self->set_from_path ($path);
	$self->set_max_headlines ($default_number);
    }
    
    return $self;
}


##
# $self->title;
# -------------
#   Returns the title of that component
##
sub title
{
    my $self = shift;
    return $self->{title};
}


##
# $self->set_title ($new_title);
# ------------------------------
#   $new_title - new title to set
##
sub set_title
{
    my $self = shift;
    $self->{title} = shift;
    $self->{title} = join ' ', split /(?:\n|\r)/, $self->{title};
    $self->{title} =~ s/^\s+//;
    $self->{title} =~ s/\s+$//;
}


##
# $self->from_path;
# -----------------
#   Returns the from_path of that component
##
sub from_path
{
    my $self = shift;
    return $self->{from_path};
}


##
# $self->set_from_path ($new_from_path);
# --------------------------------------
#   $new_from_path - new from_path to set
##
sub set_from_path
{
    my $self = shift;
    $self->{from_path} = shift;
    $self->{from_path} = join ' ', split /(?:\n|\r)/, $self->{from_path};
    $self->{from_path} =~ s/^\s+//;
    $self->{from_path} =~ s/\s+$//;
}


sub max_headlines
{
    my $self = shift;
    return $self->{max_headlines} || DEFAULT_MAX_HEADLINES;
}


sub set_max_headlines
{
    my $self = shift;
    $self->{max_headlines} = shift || '';
    $self->{max_headlines} = join ' ', split /(?:\n|\r)/, $self->{max_default_headlines} || '';
    $self->{max_headlines} =~ s/^(?:\s|\+)+//;
    $self->{max_headlines} =~ s/\s+$//;
    $self->{max_headlines} = DEFAULT_MAX_HEADLINES
        unless ($self->{max_headlines} =~ /^\+?\d+$/);
}


sub max_personalized_headlines
{
    my $self = shift;
    return $self->{max_headlines};
}


sub max_default_headlines
{
    my $self = shift;
    return $self->{max_headlines};
}


##
# $self->default_headlines;
# -------------------------
#   Returns the default headlines, with no personalization as
#   the user is not logged in.
##
sub default_headlines
{
    my $self = shift;
    
    # performs the query and put results in the stash
    my $document_t = flo::Standard::table ('Document');
    my $query = $document_t->select ( {
	cols  => '*',
	where => { Full_Path => $self->from_path() },
	sort  => [ qw /Date_Created/ ],
	desc  => 1,
	page  => [ 1, $self->max_default_headlines ],
    } );
    
    return [ $query->fetch_all ];
}


##
# $self->user_logged_in;
# ----------------------
#   Returns TRUE if a user is logged in, FALSE otherwise.
##
sub is_user_logged_in
{
    return flo::Standard::current_user();
}


##
# $self->personalized_headlines;
# ------------------------------
#   Returns the personalized headlines.
#   If the user is not logged in, returns an empty list.
##
sub personalized_headlines
{
    my $self = shift;
    my $user = flo::Standard::current_user() || return [];
    
    $self->{_personalized_headlines} ||= do {
	# This is horrible, but I really don't see how to get around it
	# until we get the RSS search engine
	my $sql  = <<SQL;
SELECT DISTINCT Document.*
FROM   Document, Document_Audience, Audience, Preference_Audience, Editor, Preference_Language
WHERE
        -- join the tables together
        (
                Preference_Language.Language_ID = Document.Lang         AND
                Preference_Language.Editor_ID = Editor.ID               AND
                Preference_Audience.Audience_ID = Audience.ID           AND
                Preference_Audience.Editor_ID = Editor.ID               AND
                Document_Audience.Audience_ID = Audience.ID             AND
                Document_Audience.Document_ID = Document.ID
        )
AND
        -- limit to the specified path
        Full_Path LIKE ?
AND
        -- limit to the current editor
        Editor.Login = ?
AND
        (
                -- get the wanted languages
                Preference_Language.Language_ID = Document.Lang
        OR
                -- get the wanted audiences
                Preference_Audience.Value       = 1
        )
AND
        -- remove languages which are not wanted
        Preference_Language.Value = 1
ORDER BY Date_Created DESC
SQL
	
	$sql .= "\nLIMIT 0, " . $self->max_personalized_headlines;
	my $dbh = lib::sql::DBH->get;
	my $sth = $dbh->prepare_cached ($sql);
	$sth->execute ($self->from_path . '_%', $user->login);
	my $query = new lib::sql::Query (sth => $sth, bless_into => 'flo::Record::Document');
	my @res = $query->fetch_all;
	\@res;
    };
    
    my $res = $self->{_personalized_headlines};
    return wantarray ? @{$res} : $res;
}


##
# $self->html; (DEPRECATED)
# ------------
#   Returns an HTML view of this component
##
sub html
{
    my $self = shift;
    return 'Headlines [' . $self->from_path . ' (' . $self->max_headlines . ') ]';
}


sub to_html { return shift->html }


1;
