# -------------------------------------------------------------------------------------
# flo::editor::Photo
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden.
#
#    Description:
#
#      Image content component. Should let the user upload images
#
# -------------------------------------------------------------------------------------
package flo::editor::Photo;
use MKDoc::Util::Text2HTML;
use MKDoc::XML::Dumper;
use MKDoc::Config;
use flo::Standard qw /cgi path_info raw_path_info session table template current_document/;
use flo::Editor;
use strict;
use Carp;

use base qw /flo::Component
	     flo::editor::Mixin::compute_name
	     flo::editor::Mixin::normalize_name
	     flo::editor::Mixin::upload_file
	     flo::editor::Mixin::hr_name
	     /;

sub activate
{
    my $self = shift;
    return unless ($self->SUPER::activate (@_));
    eval "use Image::Magick;";
    return if (defined $@ and $@);
    return 1;
}


##
# $self->internal_location();
# ---------------------------
# Returns the internal location associated with $self.
##
sub internal_location
{
    my $self = shift;
    return MKDoc::Config->IMAGE_HTTP_PATH . "/" . $self->{image};
}


sub internal_location_scaled
{
    my $self = shift;
    my $path = $self->internal_location;
    my $filename;    
    ($path, $filename) = $path =~ /(.*)\/(.*)/;
    $path .= "/scaled/$filename";
    return $path;
}


sub internal_location_small
{
    my $self = shift;
    my $path = $self->internal_location;
    my $filename;
    ($path, $filename) = $path =~ /(.*)\/(.*)/;
    $path .= "/thumbnail/$filename";
    return $path;
}


sub preferred_extension
{
    my $self  = shift;
    my $file  = $self->{image} || return 'jpg';
    $file =~ s/^.*\///;
    my ($ext) = $file =~ /.*?\.(.*)/;
    $ext ||= $self->type();
    return $ext;
}


sub new
{
    my $class = shift;
    $class    = ref $class || $class;
    my $self  = bless { @_ }, $class;
    
    my $cgi = $self->{cgi};
    my $param_name = $self->{param_name};   
    
    # if CGI is defined, then we probably want to do some stuff
    if (defined $cgi)
    {
	# update alt field
	if (defined $cgi->param ($param_name . "_alt"))
	{
	    my $alt = $cgi->param ($param_name . "_alt");
	    my $description = $cgi->param ($param_name . "_description");
	    my $creator = $cgi->param ($param_name . "_creator") || $class->_current_editor_name;
	    my ($created_day, $created_month, $created_year) =
		$class->_fix_date ( $cgi->param ($param_name . "_created_day"),
				    $cgi->param ($param_name . "_created_month"),
				    $cgi->param ($param_name . "_created_year") );
	    
	    my $rights = $cgi->param ($param_name . "_rights");
	    my $coverage = $cgi->param ($param_name . "_coverage");
	    $self->{alt} = $alt;
	    $self->{description} = $description;
	    $self->{creator} = $creator;
	    $self->{created_day} = $created_day;
	    $self->{created_month} = $created_month;
	    $self->{created_year} = $created_year;
	    $self->{rights} = $rights;
	    $self->{coverage} = $coverage;
	}
	else
	{
	    my $empty = "";
	    $self->{alt} = $empty;
	    $self->{alt} = $empty;
	    $self->{description} = $empty;
	    $self->{created_day} = $empty;
	    $self->{created_month} = $empty;
	    $self->{created_year} = $empty;
	    $self->{rights} = $empty;
	    $self->{coverage} = $empty;
	    
	    my $creator = $class->_current_editor_name;
	    $self->{creator} = $creator;
  	}
	
	# if the upload field is not empty, then the user
	# just uploaded a new image
	if (defined $cgi->param ($param_name . "_upload"))
	{
	    my $new_image = $cgi->param ($param_name);
	    if ($new_image)
	    {
		my $new_image_no_slash  = $new_image;
		$new_image_no_slash =~ s/^.*\///g;
		$new_image_no_slash =~ s/^.*\\//g;
		
		my $new_image_friendly  = $self->normalize_name ($new_image_no_slash);
		
		my $dir           = $self->compute_name;
		my $dir_thumbnail = "$dir/thumbnail";
		my $dir_scaled    = "$dir/scaled";
		
		my $full_dir           = MKDoc::Config->IMAGE_DISK_PATH . "/" . $dir;
		my $full_dir_thumbnail = MKDoc::Config->IMAGE_DISK_PATH . "/" . $dir_thumbnail;
		my $full_dir_scaled    = MKDoc::Config->IMAGE_DISK_PATH . "/" . $dir_scaled;
		
		mkdir $full_dir           or confess "Cannot create $dir";
		mkdir $full_dir_thumbnail or confess "Cannot create $full_dir";
		mkdir $full_dir_scaled    or confess "Cannot create $dir_scaled";
		
		$class->upload_file ('image_disk_path', $dir, $new_image_friendly, $cgi->param ($param_name))
		    or return $self;

		
		# we uploaded the photo, now we need to make the scaled
		# and thumbnailed versions of the image
		eval {
		    my $config = new MKDoc::Config ('flo.plugin.Photo');
		    my $scaled_max_x = $config->get ('scaled_max_x');
		    my $scaled_max_y = $config->get ('scaled_max_y');
		    my $thumb_max_x  = $config->get ('thumb_max_x');
		    my $thumb_max_y  = $config->get ('thumb_max_y');
		    
		    my $image = new Image::Magick;
		    my $retcode = $image->Read ("$full_dir/$new_image_friendly");
		    die $retcode if (defined $retcode and $retcode =~ /exception/i);
		    
		    $image->Resize (geometry => $scaled_max_x . 'x' . $scaled_max_y);
		    $image->Write ("$full_dir_scaled/$new_image_friendly");
		    $image->Resize (geometry => $thumb_max_x  . 'x' . $thumb_max_y);
		    $image->Write ("$full_dir_thumbnail/$new_image_friendly");
		    
		    $self->{image} = "$dir/$new_image_friendly";
		};
		if (defined $@ and $@)
		{
		    carp ("Cannot upload image: $@");
		    $self->{image} = '';
		}

		$self->{alt} =
		    $cgi->param ($param_name . "_alt")    ||
		    $class->hr_name ($new_image_friendly) ||
		    'image_title';
	    }
	}
	else
	{
	    $self->{image} = $cgi->param ($param_name . "_img");
	}
    }
    
    return $self;
}


##
# _fix_date ($day, $month, $year);
# --------------------------------
#   Fix the date if it doesn't make sense
##
sub _fix_date
{
    my $class = shift;
    my $day = shift;
    my $month = shift;
    my $year = shift;
    
    $day ||= '';
    $day   =~ /^\d+/ or $day = 1;

    $month ||= '';
    $month =~ /^\d+/ or $month = 1;
    
    my $localtime = [ localtime ];
    my $current_year = $localtime->[5] + 1900;    
    $year ||= '';
    $year = $current_year unless $year =~ /^[+-]?\d+$/;
    
    $month > 0 and $month < 13 or $month = 1;
    
    $day = 1  if ($day < 1);
    $day = 28 if ( ( ($year % 4) and ($month == 2) ) and ($day > 28) );
    $day = 29 if ( ( ($year %4 == 0) and ($month == 2) ) and ($day > 29) );
    $day = 30 if ( ( ($month == 4) or ($month == 6) or
		     ($month == 9) or ($month == 11) ) and $day > 30 );
    $day = 31 if ( ( ($month == 1) or ($month == 3)  or
		     ($month == 5) or ($month == 7)  or
		     ($month == 8) or ($month == 10) or
		     ($month == 12) ) and $day > 31 );
    
    return ($day, $month, $year);
}


##
# _current_editor_name;
# ---------------------
#   Returns the name of the current editor
##
sub _current_editor_name
{
    my $class = shift;
    my $session = session();
    my $user_id = $session->{'Editor_ID'};
    return 'nobody' unless (defined $user_id and $user_id);
    my $editor_table = table ('Editor');
    my $editor = $editor_table->get ($user_id);
    return 'nobody' unless (defined $editor);
    return "$editor->{First_Name} $editor->{Family_Name}"
	if ( defined $editor->{First_Name} and $editor->{First_Name} and
	     defined $editor->{Family_Name} and $editor->{Family_Name} );
    return $editor->{Family_Name};
}


sub uri_small
{
    my $self  = shift;
    my $small = MKDoc::Config->PHOTO_SMALL_HINT;
    return $self->is_saved() ?
        $self->SUPER::uri (@_) . ",$small" :
	$self->Thumbnail_URI (@_);
}


sub uri_scaled
{
    my $self = shift;
    my $scaled = MKDoc::Config->PHOTO_SCALED_HINT;
    return $self->is_saved() ?
        $self->SUPER::uri (@_) . ",$scaled" :
	$self->Scaled_URI (@_);
}


sub uri_html
{
    my $self = shift;
    return $self->SUPER::uri (@_) . ",html";
}


sub uri
{
    my $self = shift;
    return $self->is_saved() ?
        $self->SUPER::uri (@_) :
	$self->Original_URI (@_);
}


sub Thumbnail_URI
{
    my $self = shift;
    my $file = $self->{image};
    return unless (defined $file);
    
    my @path = split /(?:\/|\\)/, $file;
    my $filename = pop (@path);
    push @path, 'thumbnail';
    push @path, $filename;
    my $new_file = join '/', @path;
    return MKDoc::Config->IMAGE_HTTP_PATH . "/" . $new_file;
}


sub Scaled_URI
{
    my $self = shift;
    my $file = $self->{image};
    return unless (defined $file);
    
    my @path = split /(?:\/|\\)/, $file;
    my $filename = pop (@path);
    push @path, 'scaled';
    push @path, $filename;
    my $new_file = join '/', @path;
    return MKDoc::Config->IMAGE_HTTP_PATH . "/" . $new_file;
}


sub Original_URI
{
    my $self = shift;
    my $file = $self->{image};
    return MKDoc::Config->IMAGE_HTTP_PATH . "/" . $file;
}


sub URI
{
    my $self = shift;
    return $self->DC_Identifier;
}


sub title
{
    my $self = shift;
    return $self->{alt};
}


sub DC_Identifier
{
    my $self = shift;
    # dangerous
    my $path_info = $self->Original_URI;
    my $cgix = cgi->new;
    $cgix->path_info ($path_info);
    return $cgix->self_url;
}


sub generate_xml
{
    my $self = shift;
    my $cgi  = delete $self->{cgi};
    my $res = pl2xml ($self);
    $self->{cgi} = $cgi;
    return $res;
}


sub parse_xml
{
    my $self = shift;
    my $class = ref $self;
    my $data = shift;
    if ($data)
    {
	if ($data =~ /<img src/)
	{
	    my ($img, $alt) = $data =~ /<img src=\"(.*?)\" alt=\"(.*)\" \/>/;
	    $self->{image}  = $img;
	    $self->{alt}    = html2text ($alt) || $self->{image};
	}
	else
	{
	    my $hash = xml2pl ($data);
	    for (keys %{$hash}) { $self->{$_} = $hash->{$_} }
	}
    }
    else
    {
	my $empty = "";
	# u_mark (\$empty);
	$self->{alt} = $empty;
	$self->{alt} = $empty;
	$self->{description} = $empty;
	$self->{created_day} = $empty;
	$self->{created_month} = $empty;
	$self->{created_year} = $empty;
	$self->{rights} = $empty;
	$self->{coverage} = $empty;
	
	my $creator = $class->_current_editor_name;
	# u_mark (\$creator);
	$self->{creator} = $creator;
    }
}


sub html
{
    my $self = shift;
    return '<img src="' . $self->Thumbnail_URI . '" alt="' . text2html ($self->{alt}) . '" />';
}


1;

