# -------------------------------------------------------------------------------------
# flo::plugin::Admin::Properties
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver <jhiver@mkdoc.com>
#    Copyright : (c) MKDoc Holdings Ltd, 2002
#
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden
#
#    Description:
#
# -------------------------------------------------------------------------------------
package flo::plugin::Admin::Properties;
use MKDoc::Util::TemplateFinder;
use MKDoc::Config;
use MKDoc::Error;
use flo::Standard;
use flo::RedirectManager;
use strict;
use warnings;
use 5.008_000;
use Carp;

use base qw /flo::Plugin/;

sub template_path { '/admin/properties' }


##
# $class->templates;
# ------------------
# Returns a structure which can be used within a template to choose
# a document template from a select box.
#
# See MKDoc::Util::TemplateFinder::list_structure()
##
sub templates
{
    my $self    = shift;
    my $default = $self->parent()->template() || shift || 'default';
    return MKDoc::Util::TemplateFinder::list_structure ('document', $default);
}


##
# $self->selected_template;
# -------------------------
# Returns the template which is currently selected.
#
# See MKDoc::Util::TemplateFinder::list_structure_selected()
##
sub selected_template
{
    my $self = shift;
    my $selected = MKDoc::Util::TemplateFinder::list_structure_selected ($self->templates());
    $selected ||= do {
	my $name  = $self->parent->popular_children_template;
	my $label = $name;
	$label =~ s/_/ /g;
	{
	    value    => $name,
	    label    => $label,
	    selected => 'selected',
	};
    };
    
    return $selected;
}


##
# $self->unselected_templates;
# ----------------------------
# Returns the templates which are not currently selected.
#
# See MKDoc::Util::TemplateFinder::list_structure_unselected()
##
sub unselected_templates
{
    my $self = shift;
    return MKDoc::Util::TemplateFinder::list_structure_unselected ($self->templates());
}


##
# $class->cache_controls;
# -----------------------
#   Returns a structure which can be used within a template to choose
#   a document cache control value from a select box.
##
sub cache_controls
{
    my $self = shift;
    my $doc  = flo::Standard::current_document();
    my $cgi  = flo::Standard::cgi();
    
    my $selected_cache_control = $cgi->param ('cache_control');
    $selected_cache_control    = $doc->cache_control unless (defined $selected_cache_control);
    
    my @res = map {
	{
	    value    => $_,
	    label    => $_,
	    selected => ($_ == $selected_cache_control),
	}
    } $doc->available_cache_controls;
    
    return wantarray ? @res : \@res;
}


##
# $self->selected_cache_control;
# ------------------------------
#   Returns the cache_control which is currently selected
##
sub selected_cache_control
{
    my $self = shift;
    my ($selected) = map { ($_->{selected}) ? $_ : () } $self->cache_controls (@_);
    $selected ||= do {
	{
	    value => $self->cache_control,
	    label => $self->cache_control,
	    selected => 'selected',
	};
    };
    
    return $selected;
}


##
# $self->unselected_cache_controls;
# ---------------------------------
#   Returns the cache_controls which are currently not selected
##
sub unselected_cache_controls
{
    my $self = shift;
    $self = ref $self || $self;
    my @unselected = map { ($_->{selected}) ? () : $_ } $self->cache_controls (@_);
    return (wantarray) ? @unselected : \@unselected;
}


##
# $class->langs;
# --------------
#   Returns a structure which can be used within a template to choose
#   a document cache control value from a select box.
##
sub langs
{
    my $self = shift;
    my $doc  = $self->parent;
    my $cgi  = flo::Standard::cgi();
    
    my $current_lang = $cgi->param ('lang') || $doc->lang;
    my $languages_rtl = MKDoc::Config->parsefile (MKDoc::Config->LANGUAGE_LIST_RTL);
    my @res = map {
	{
	    value          => $_,
	    label          => $doc->lang_label ($_),
	    selected       => ($_ eq $current_lang),
	    direction      => (exists $languages_rtl->{$_}) ? 'rtl'   : 'ltr',
	    align          => (exists $languages_rtl->{$_}) ? 'right' : 'left',
	    align_opposite => (exists $languages_rtl->{$_}) ? 'left'  : 'right',
	}
    } $doc->available_langs;
    
    return wantarray ? @res : \@res;
}


##
# $self->selected_cache_control;
# ------------------------------
#   Returns the language which is currently selected
##
sub selected_lang
{
    my $self = shift;
    my ($selected) = map { ($_->{selected}) ? $_ : () } $self->langs (@_);
    $selected ||= do {
	my $languages_rtl = MKDoc::Config->parsefile (MKDoc::Config->LANGUAGE_LIST_RTL);
	my $val = $self->parent->lang;
	{
	    value => $val,
	    label => $self->parent->lang_label ($val),
	    selected => 'selected',
	    direction      => (exists $languages_rtl->{$val}) ? 'rtl'   : 'ltr',
	    align          => (exists $languages_rtl->{$val}) ? 'right' : 'left',
	    align_opposite => (exists $languages_rtl->{$val}) ? 'left'  : 'right',
	}
    };
    return $selected;
}


##
# $self->unselected_langs;
# ------------------------
#   Returns the langs which are currently not selected
##
sub unselected_langs
{
    my $self = shift;
    $self = ref $self || $self;
    my @unselected = map { ($_->{selected}) ? () : $_ } $self->langs (@_);
    return (wantarray) ? @unselected : \@unselected;
}


##
# $self->audiences;
# -----------------
#   Returns all audiences related to the current document,
#   but adds a 'checked' flag for those which are checked.
##
sub audiences
{
    my $self = shift;
    my $document = flo::Standard::current_document();
    
    my $checked = { map { $_->id() => 1 } $document->audiences };
    my $audience_t = flo::Standard::table ('Audience');
    my @audiences = $audience_t->select ( sort => [ qw /Label/ ] )->fetch_all;
    $_->{checked} = ($checked->{$_->id()}) ? 'checked' : undef for (@audiences);
    
    return wantarray ? @audiences : \@audiences;
}


##
# $self->activate;
# ----------------
#   Returns TRUE if this plugin can be activated, FALSE
#   otherwise.
##
sub activate
{
    my $self = shift;
    return unless ($self->SUPER::activate (@_));
    my $current_document = flo::Standard::current_document();
    return $current_document->Can_Admin;
}


##
# $self->http_get;
# ----------------
# Displays the form which lets the editor choose which audiences
# the current document relates to.
##
sub http_get
{
    my $self = shift;
    $self->render_http (
	self       => $self,
	object     => flo::Standard::current_document(),
	__input__  => 'XML',
	__output__ => 'XHTML',
       );
    
    return 'TERMINATE';
}


##
# $self->http_post;
# -----------------
#   Reflect the changes on the database and redirects to
#   current document.
##
sub http_post
{
    my $self = shift;
    my $cgix = flo::Standard::cgi()->new;
    
    my $document_t = flo::Standard::table ('Document');
    my $document = $self->parent;
    
    # let's set the document object attributes
    $document->set_title         ($cgix->param ( 'title'         ) );
    $document->set_description   ($cgix->param ( 'description'   ) );
    $document->set_keywords      ($cgix->param ( 'keywords'      ) );
    $document->set_lang          ($cgix->param ( 'lang'          ) );
    $document->set_rights        ($cgix->param ( 'rights'        ) );
    $document->set_publisher     ($cgix->param ( 'publisher'     ) );
    $document->set_template      ($cgix->param ( 'template'      ) );
    $document->set_cache_control ($cgix->param ( 'cache_control' ) );
    
    # we want to set the document name unless it's the root document
    if ($document->parent)
    {
	my $name = $cgix->param ( 'name' );
	flo::RedirectManager->document_renamed ($document, $name);
	$document->set_name ($name);
    }
    
    $document->validate() || return $self->http_get();
    $document->save();
    $self->_set_audiences_from_cgi ($document);
    
    # we have finished changing the properties, let's redirect
    # to the current document
    $cgix->delete ($_) for ($cgix->param);
    $cgix->path_info ($document->path);
    print $cgix->redirect ($cgix->self_url);    
    return 'TERMINATE';
}


##
# $self->_set_audiences_from_cgi;
# -------------------------------
#   Sets the audiences in the 'Audience' table from the CGI object.
##
sub _set_audiences_from_cgi
{
    my $self = shift;
    my $document = shift;
    my $cgix = flo::Standard::cgi->new;
    
    my $document_audience_table = flo::Standard::table ('Document_Audience');
    my $audience_table = flo::Standard::table ('Audience');
    my @submited_audiences = $cgix->param ('audience');
    
    my %existing_audiences = map { $_->id() => 1 } $audience_table->search()->fetch_all();
    my @valid_ids = ();
    foreach my $audience_id (@submited_audiences)
    {
	next unless $existing_audiences{$audience_id};
	push @valid_ids, $audience_id;
    }
    
    $document_audience_table->delete ( Document_ID => $document->id() );
    $document_audience_table->insert ( Document_ID => $document->id(), Audience_ID => $_ ) for (@valid_ids);
}


##
# $self->lang;
# ------------
#   Returns the currently selected language,
#   or the language of the parent if none is selected.
##
sub lang
{
    my $self = shift;
    my $current = $self->selected_lang;
    return $current->{value} if (defined $current);
    return $self->SUPER::lang;
}


##
# $self->form_input ($name);
# --------------------------
# Returns the default value to use for the form of
# name $name.
##
sub form_input
{
    my $self   = shift;
    my $param  = shift;
    my $object = flo::Standard::current_document();
    my $cgix   = flo::Standard::cgi()->new();
    return $cgix->param ($param) || $object->$param();
}


1;
