package lib::cgi::Session;
use lib::Exception;
use MKDoc::CGI;
use lib::sql::Table;
use CGI::Carp;
use strict;
use vars qw /$AUTOLOAD/;

use constant MAX_TIMEOUT   => 2 * 3600;
use constant KEY_LENGTH    => 50;
use constant TIMEOUT_CHECK => 1;
use constant IP_CHECK      => 0;

$::MKD_lib_cgi_Session_ETERNAL = undef;
sub _ETERNAL_()      { return $::MKD_lib_cgi_Session_ETERNAL  }
sub _SET_ETERNAL_($) { $::MKD_lib_cgi_Session_ETERNAL = shift }


##
# __PACKAGE__->new_from_http();
# -----------------------------
#   Builds a new lib::cgi::Session object, but based on HTTP
#   authentication rather than cookies
##
sub _new_from_http
{
    my $class = shift;
    $class = ref $class || $class;
    
    my $session_table = lib::sql::Table->table ('Session') or
	confess "ERROR: 'Session' table does not seem to exist";
    
    my $user_table    = lib::sql::Table->table ('Editor') or
	confess "ERROR: 'Editor' table does not seem to exist";

    # retrieves the user info and checks if the user exists
    my $user = $user_table->search ( 'Login' => $ENV{REMOTE_USER} )->next;
    (defined $user) or
	confess "NO_SUCH_USER: $ENV{REMOTE_USER} does not exist";
    
    # creates the session objects, inserts its info into
    # the database and returns it
    my $session_id = join '', map { chr int rand (26) + ord 'A' } 1..KEY_LENGTH;
    my $session_time = time;
    
    # first of all, delete any previous session for that user
    $session_table->delete ( 'Editor_ID' => $user->{'ID'} );
    
    $session_table->insert ( 'ID'        => $session_id,
			     'Last_Time' => $session_time,
			     'IP'        => $ENV{REMOTE_ADDR},
			     'Editor_ID' => $user->{'ID'} );
    
    # let us merge user and session attributes
    my $res = {};
    for my $key (keys %{$user}) { $res->{'Editor'    . '_' . $key} = $user->{$key} }
    $res->{'Session' . '_' . 'ID'}      = $session_id;
    $res->{'Session' . '_' . 'Last_Time'}    = $session_time;
    $res->{'Session' . '_' . 'IP'}      = $ENV{REMOTE_ADDR};
    $res->{'Session' . '_' . 'Editor_ID'} = $user->{'ID'};
    $res->{cookie_name} = 'none';
    $res->{'.using_http'} = 1;
    _SET_ETERNAL_ (bless $res, $class);
    return _ETERNAL_;
}


##
# __PACKAGE__->new ($cookie_name, $login, $password);
# __PACKAGE__->new ($cookie_name);
##
sub new
{
    my $class       = shift;
    $class          = ref $class || $class;
    
    # if the session has been instanciated already,
    # returns the object.
    _ETERNAL_ and return _ETERNAL_;
    
    return $class->_new_from_http() if (defined $ENV{REMOTE_USER} and $ENV{REMOTE_USER});
    
    my $cookie_name = shift or
	confess "BAD_ARGS: First argument should be a cookie name";
    
    my $session_table = lib::sql::Table->table ('Session') or
	confess "ERROR: 'Session' table does not seem to exist";
    
    my $user_table    = lib::sql::Table->table ('Editor') or
	confess "ERROR: 'Editor' table does not seem to exist";
    
    if (@_ == 0)
    {
	# retrieve info from the web
	my $cgi = MKDoc::CGI->get;

	my $session_id = $cgi->cookie ($cookie_name) or
	    confess "NO_COOKIE: The browser sent no session cookie related info";
	
	# retrieve session info from the database
	my $session = $session_table->search ('ID' => $session_id )->next or
	    confess "NOT_IN_DB: That session did not seem to be in the database";
	
	# make sure that this session is valid
	(defined $session) or return;
	if (IP_CHECK)
	{
	    ($session->{'IP'} eq $ENV{REMOTE_ADDR} ) or
		confess "UNVALID_IP: Session IP is $session->{IP} whilst current IP is $ENV{REMOTE_ADDR}";
	}
	if (TIMEOUT_CHECK)
	{
	    ($session->{'Last_Time'} + MAX_TIMEOUT > time) or
		confess ( "SESSION_EXPIRED: Maximum time was $session->{Last_Time} + " . MAX_TIMEOUT .
			  " whilst current time is " . time );
	}
	
	# update session time
	$session->{'Last_Time'} = time;
	$session_table->modify ($session);
	
	# if it is, retrieve user info which is associated with it
	my $user = $user_table->search ( 'ID' => $session->{'Editor_ID'} )->next or
	    confess "NO_SUCH_USER: The user that this session references does not seem to exist";
	
	# let us merge user and session attributes
	my $res = {};
	for my $key (keys %{$user})    { $res->{'Editor'    . '_' . $key} = $user->{$key}    }
	for my $key (keys %{$session}) { $res->{'Session' . '_' . $key} = $session->{$key} }
	$res->{cookie_name} = $cookie_name;
	
	_SET_ETERNAL_ (bless $res, $class);
	
	# Clean up the session table by removing timed out session
	my $condition = new lib::sql::Condition;
	$condition->add ('Last_Time', '<', time - MAX_TIMEOUT);
	$session_table->delete ($condition);
	
	$res->{cookie_name} = $cookie_name;   
	
	return _ETERNAL_;
    }
    else
    {
	# Clean up the session table by removing timed out session
	my $condition = new lib::sql::Condition;
	$condition->add ('Last_Time', '<', time - MAX_TIMEOUT);
	$session_table->delete ($condition);
	
	my $login = shift or
	    confess "LOGIN_IS_UNDEF: The user login is not defined";
	
	my $password = shift or
	    confess "PASSWORD_IS_UNDEF: The user password is not defined";
	
	# retrieves the user info and checks if the user exists.
	my $user = $user_table->search ( 'Login' => $login )->next;
	(defined $user) or
	    confess "NO_SUCH_USER: $login does not exist";
	
	# checks that the user password is correct
	($user->{'Password'} eq $password) or
	    confess "INCORRECT_PASSWORD: The password which is provided is not correct";	
	
	# creates the session objects, inserts its info into
	# the database and returns it.
	my $session_id = join '', map { chr int rand (26) + ord 'A' } 1..KEY_LENGTH;
	my $session_time = time;
	
	# first of all, delete any previous session for that user
	$session_table->delete ( 'Editor_ID' => $user->{'ID'} );
	
	$session_table->insert ( 'ID'        => $session_id,
				 'Last_Time' => $session_time,
				 'IP'        => $ENV{REMOTE_ADDR},
				 'Editor_ID' => $user->{'ID'} );

	# let us merge user and session attributes
	my $res = {};
	for my $key (keys %{$user})    { $res->{'Editor'    . '_' . $key} = $user->{$key}    }
	$res->{'Session' . '_' . 'ID'}      = $session_id;
	$res->{'Session' . '_' . 'Last_Time'}    = $session_time;
	$res->{'Session' . '_' . 'IP'}      = $ENV{REMOTE_ADDR};
	$res->{'Session' . '_' . 'Editor_ID'} = $user->{'ID'};
	$res->{cookie_name} = $cookie_name;   

	_SET_ETERNAL_ (bless $res, $class);
	return _ETERNAL_;
    }
}


##
# uses_cookie();
# --------------
#   Returns TRUE if the current session object uses cookies,
#   FALSE otherwise.
##
sub uses_cookie
{
    my $self = shift;
    return defined $self->{'.using_http'} and $self->{'.using_http'};
}


##
# $obj->header;
# -------------
#   Prints the html header to store the proper cookie
#   into the user's browser.
##
sub header
{
    my $self = shift;
    my $cgi  = MKDoc::CGI->get;
    return $cgi->header (-cookie => $cgi->cookie ( -name => $self->{cookie_name}, -value => $self->{ID}) );
}


##
# Accessor function for the package attributes :
# ----------------------------------------------
# user_id     user_login    user_password
# session_id  session_time  session_ip
##
sub AUTOLOAD
{
    my $self = shift;
    my ($attrib) = $AUTOLOAD =~ /.*::(.*)/;
    $self->{$attrib} = shift || return $self->{$attrib};
}


1;
