#!/usr/bin/perl
use strict;
use warnings;
use MKDoc::Handler::Initialize;
use flo::Standard;
use MKDoc::Util::Text2HTML;
use MKDoc::Stemmer;

our $INDEXABLE_RECENTNESS = shift || die "Usage: $0 <indexable_recentness_in_seconds>";
$ENV{MKDOC_DIR} || die 'MKDOC_DIR is not defined!';

main();
sub main
{
    defined $ENV{SITE_DIR} and return do {
        MKDoc::Handler::Initialize::handler();
        index_remove();
        index_update();
        index_insert();
    };
    
    foreach my $mkdoc_site (site_list())
    {
        print "INDEXING: $mkdoc_site\n";
        $ENV{SITE_DIR} = $mkdoc_site;
        main();
    }
}


##
# site_list;
# ----------
# Returns every registered domain starting with 'www'.
##
sub site_list
{
    my $file = $ENV{MKDOC_DIR} . '/conf/sites.conf';
    open FP, "<$file" || die "Cannot read-open $file!";
    my $data = join '', <FP>;
    close FP;
    
    my $VAR1;
    eval $data || die $@;

    my %set = ();
    for (values %{$VAR1}) { $set{$_} = 1 }
    return sort keys %set;
}


##
# index_remove;
# -------------
#   Remove keywords indexing documents that don't exist
##
sub index_remove
{
    my $document = flo::Standard::table ('Document');
    my $index = flo::Standard::table ('Document_Index');
    
    # we have to find all ids which exist in index
    # and which don't exist in $document anymore.
    my @document_ids = map { $_->{ID} => 1 } $document->select ( cols => [ qw /ID/ ] )->fetch_all;
    my %index_ids = map { $_->{Record_ID} => 1 } $index->select ( cols => [ qw /Record_ID/ ] )->fetch_all;
    
    foreach my $id (@document_ids)
    {
	delete $index_ids{$id};
    }
    
    foreach my $id (keys %index_ids)
    {
	print "\t[delete] $id\n";
	$index->delete ( Record_ID => $id );
    }
}


##
# index_update;
# -------------
#   Indexes documents which were modified recently
##
sub index_update
{
    my $document = flo::Standard::table ('Document');
    my $index = flo::Standard::table ('Document_Index');
    my $date = reindex_date();
    
    my $condition = new lib::sql::Condition;
    $condition->add ( 'Date_Last_Modified', '>', $date );
    my $query = $document->search ( $condition );
    
    while (my $doc = $query->next)
    {
	print "\t[update] $doc->{Full_Path}\n";
	$index->delete ( Record_ID => $doc->{ID} );
	$document->lang ($doc->{Lang});	
	
	foreach my $col (sort keys %{$doc})
	{
	    next unless (defined $document->{weight}->{$col});
	    
	    my %keywords = map { $_ => 1 } _data_split ($document, $doc->{$col});
	    my @keywords = keys %keywords;
	    
	    foreach my $keyword (@keywords)
	    {
		$index->insert ( Record_ID => $doc->{ID},
				 Column_Name => $col,
				 Keyword => $keyword );
	    }
	}
    }
}


##
# index_insert;
# -------------
#   Adds keywords for documents which are being added
##
sub index_insert
{
    my $document = flo::Standard::table ('Document');
    my $index = flo::Standard::table ('Document_Index');
    
    # we have to find all ids which exist in @document
    # and which have no reference in index
    my @document_ids = map { $_->{ID} } $document->select ( cols => [ qw /ID/ ] )->fetch_all;
    my %index_ids = map { $_->{Record_ID} => 1 } $index->select ( cols => [ qw /Record_ID/ ] )->fetch_all;
    
    foreach my $id (@document_ids)
    {
	(defined $index_ids{$id}) and next;
	
	my $doc = $document->get ($id) or
	    (Carp::cluck "Cannot get document with ID: $id" and next);
	
	$document->lang ($doc->{Lang});	
	print "\t[insert] $doc->{Full_Path}\n";
	
	foreach my $col (sort keys %{$doc})
	{
	    next unless (defined $document->{weight}->{$col});
	    
	    my %keywords = map { $_ => 1 } _data_split ($document, $doc->{$col});
	    my @keywords = keys %keywords;
	    
	    foreach my $keyword (@keywords)
	    {
		$index->insert ( Record_ID => $id,
				 Column_Name => $col,
				 Keyword => $keyword );
	    }
	}
    }
}


##
# reindex_date;
# -------------
#   Returns the reindex date given $INDEXABLE_RECENTNESS
##
sub reindex_date
{
    # prefill date fields with current date
    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime (time - $INDEXABLE_RECENTNESS);
    $mon++;
    $year += 1900;
    
    if (length ($mon)  == 1) { $mon  = "0" . $mon  }
    if (length ($mday) == 1) { $mday = "0" . $mday }
    
    return "$year-$mon-$mday $hour:$min:$sec";	
}


##
# _data_split ($table, $value);
# -----------------------------
#   Splits the data into keywords, eventually depending on
#   the language which is being used
#
#   @param - $value : string that needs to be splitted
#   @returns - An array of splitted keywords
##
sub _data_split
{
    shift();
    my $value = shift;
    $value = html2text ($value);
    return MKDoc::Stemmer::split ($value);
}


1;


